/*
 * ARISTOSTLE UNIVERSITY OF THESSALONIKI
 * Copyright (C) 2015
 * Aristotle University of Thessaloniki
 * Department of Electrical & Computer Engineering
 * Division of Electronics & Computer Engineering
 * Intelligent Systems & Software Engineering Lab
 *
 * Project             : WSAT
 * WorkFile            : 
 * Compiler            : 
 * File Description    : 
 * Document Description: 
* Related Documents	   : 
* Note				   : 
* Programmer		   : RESTful MDE Engine created by Christoforos Zolotas
* Contact			   : christopherzolotas@issel.ee.auth.gr
*/


package eu.fp7.scase.wsat.inputparameter;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.CollectionTable;
import javax.persistence.ElementCollection;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import javax.persistence.Table;
import javax.persistence.Transient;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.annotations.ForeignKey;

import eu.fp7.scase.wsat.utilities.HypermediaLink;
import eu.fp7.scase.wsat.inputmessage.JavainputmessageModel;

import org.hibernate.search.annotations.Analyze;
import org.hibernate.search.annotations.Field;
import org.hibernate.search.annotations.FieldBridge;
import org.hibernate.search.annotations.Index;
import org.hibernate.search.annotations.Indexed;
import org.hibernate.search.annotations.IndexedEmbedded;
import org.hibernate.search.annotations.Store;

import eu.fp7.scase.wsat.utilities.SetStringFieldBridge;

/* This class models the data of a inputparameter resource. It is enhanced with JAXB annotations for automated representation
parsing/marshalling as well as with Hibernate annotations for ORM transformations.*/
@XmlRootElement
@Entity
@Table(name="inputparameter")
@Indexed
public class JavainputparameterModel{


    /* There follows a list with the properties that model the inputparameter resource, as prescribed in the service CIM*/
		@Column(name = "name")
		private String name;

		@ElementCollection(fetch = FetchType.EAGER)
		@CollectionTable(name="inputparameterkeywords", joinColumns=@JoinColumn(name="inputparameterId"))
		@Column(name = "keywords")
		@ForeignKey(name = "fk_inputparameter_keywords")
		@Field(index=Index.YES, analyze=Analyze.YES, store=Store.NO)
		@FieldBridge(impl=SetStringFieldBridge.class)
		private Set<String> keywords = new HashSet<String>();

		@Column(name = "type")
		private String type;

		@Id
		@GeneratedValue
		@Column(name = "inputparameterId")
		private int inputparameterId;

		// The Linklist property holds all the hypermedia links to be sent back to the client
		@Transient
		private List<HypermediaLink> linklist = new ArrayList<HypermediaLink>();

		// This property models the Many to One relationship between two resources as it is defined by the Hibernate syntax below.
		@ManyToOne(fetch = FetchType.EAGER)
		@JoinColumn(name="sourseinputparameterId")
		@ForeignKey(name = "fk_inputparameter_inputparameter")
		private JavainputparameterModel inputparameter;

		// This property models the Many to One relationship between two resources as it is defined by the Hibernate syntax below.
		@ManyToOne(fetch = FetchType.EAGER)
		@JoinColumn(name="inputmessageId")
		@ForeignKey(name = "fk_inputmessage_inputparameter")
		private JavainputmessageModel inputmessage;

		// This property models the One to Many relationship between two resources as it is defined by the Hibernate syntax below.
		@OneToMany(fetch = FetchType.EAGER, mappedBy="inputparameter",orphanRemoval=true)
		@OnDelete(action=OnDeleteAction.CASCADE)
		private Set<JavainputparameterModel> SetOfJavainputparameterModel = new HashSet<JavainputparameterModel>();

    /* There follows a list of setter and getter functions.*/
	    public void setname(String name){
        	this.name = name;
    	}

	    public void setkeywords(Set<String> keywords){
        	this.keywords = keywords;
    	}

	    public void settype(String type){
        	this.type = type;
    	}

	    public void setinputparameterId(int inputparameterId){
        	this.inputparameterId = inputparameterId;
    	}

	    public void setlinklist(List<HypermediaLink> linklist){
        	this.linklist = linklist;
    	}

		@XmlTransient
	    public void setinputparameter(JavainputparameterModel inputparameter){
        	this.inputparameter = inputparameter;
    	}

		@XmlTransient
	    public void setinputmessage(JavainputmessageModel inputmessage){
        	this.inputmessage = inputmessage;
    	}

		@XmlTransient
	    public void setSetOfJavainputparameterModel(Set<JavainputparameterModel> SetOfJavainputparameterModel){
        	this.SetOfJavainputparameterModel = SetOfJavainputparameterModel;
    	}

	    public String getname(){
        	return this.name;
    	}

	    public Set<String> getkeywords(){
        	return this.keywords;
    	}

	    public String gettype(){
        	return this.type;
    	}

	    public int getinputparameterId(){
        	return this.inputparameterId;
    	}

	    public List<HypermediaLink> getlinklist(){
        	return this.linklist;
    	}

	    public JavainputparameterModel getinputparameter(){
        	return this.inputparameter;
    	}

	    public JavainputmessageModel getinputmessage(){
        	return this.inputmessage;
    	}

	    public Set<JavainputparameterModel> getSetOfJavainputparameterModel(){
        	return this.SetOfJavainputparameterModel;
    	}


    /* This function deletes explicitly any collections of this resource that are stored in the database 
    and iteratively does so for any subsequent related resources.
    NOTE: this function is needed to handle erroneous handling of cascade delete of some hibernate versions.*/
    public void deleteAllCollections(Session hibernateSession){

        Query query = hibernateSession.createSQLQuery(String.format("DELETE FROM %s where %sId = %d","inputparameterkeywords".toLowerCase(),"inputparameter",this.getinputparameterId()));
        query.executeUpdate();

        Iterator<JavainputparameterModel> JavainputparameterModelIterator = SetOfJavainputparameterModel.iterator();
        while(JavainputparameterModelIterator.hasNext()){
            JavainputparameterModelIterator.next().deleteAllCollections(hibernateSession);
        }
    }
}
