/*
 * ARISTOSTLE UNIVERSITY OF THESSALONIKI
 * Copyright (C) 2015
 * Aristotle University of Thessaloniki
 * Department of Electrical & Computer Engineering
 * Division of Electronics & Computer Engineering
 * Intelligent Systems & Software Engineering Lab
 *
 * Project             : WSAT
 * WorkFile            : 
 * Compiler            : 
 * File Description    : 
 * Document Description: 
* Related Documents	   : 
* Note				   : 
* Programmer		   : RESTful MDE Engine created by Christoforos Zolotas
* Contact			   : christopherzolotas@issel.ee.auth.gr
*/


package eu.fp7.scase.wsat.restparameter;


import javax.ws.rs.core.UriInfo;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import com.sun.jersey.core.util.Base64;
import eu.fp7.scase.wsat.account.JavaaccountModel;

import java.util.Iterator;
import eu.fp7.scase.wsat.utilities.HypermediaLink;
import eu.fp7.scase.wsat.utilities.HibernateController;
import eu.fp7.scase.wsat.restmethod.JavarestmethodModel;

/* This class processes GET requests for restparameter resources and creates the hypermedia to be returned to the client*/
public class GetrestmethodrestparameterListHandler{


    private HibernateController oHibernateController;
    private UriInfo oApplicationUri; //Standard datatype that holds information on the URI info of this request
    private JavarestmethodModel oJavarestmethodModel;
	private String authHeader;
	private JavaaccountModel oAuthenticationAccount;

    public GetrestmethodrestparameterListHandler(String authHeader, int restmethodId, UriInfo oApplicationUri){
        this.oHibernateController = HibernateController.getHibernateControllerHandle();
        this.oApplicationUri = oApplicationUri;
        oJavarestmethodModel = new JavarestmethodModel();
        oJavarestmethodModel.setrestmethodId(restmethodId);
		this.authHeader = authHeader;
		this.oAuthenticationAccount = new JavaaccountModel(); 
    }

    public JavarestparameterModelManager getJavarestparameterModelManager(){

    	//check if there is a non null authentication header
    	if(authHeader == null){
    		throw new WebApplicationException(Response.Status.FORBIDDEN);
    	}
		else{
	    	//decode the auth header
    		decodeAuthorizationHeader();

        	//authenticate the user against the database
        	oAuthenticationAccount = oHibernateController.authenticateUser(oAuthenticationAccount);

			//check if the authentication failed
			if(oAuthenticationAccount == null){
        		throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        	}
		}

        oJavarestmethodModel = oHibernateController.getrestmethodrestparameterList(oJavarestmethodModel);
        return createHypermedia(oJavarestmethodModel);
    }

	/* This function performs the decoding of the authentication header */
    public void decodeAuthorizationHeader()
    {
    	//check if this request has basic authentication
    	if( !authHeader.contains("Basic "))
    	{
    		throw new WebApplicationException(Response.Status.BAD_REQUEST);
    	}
    	
        authHeader = authHeader.substring("Basic ".length());
        String[] decodedHeader;
        decodedHeader = Base64.base64Decode(authHeader).split(":");
        
        if( decodedHeader == null)
        {
        	throw new WebApplicationException(Response.Status.BAD_REQUEST);
        }
        
        oAuthenticationAccount.setusername(decodedHeader[0]);
        oAuthenticationAccount.setpassword(decodedHeader[1]);
    }

    /* This function produces hypermedia links to be sent to the client so as it will be able to forward the application state in a valid way.*/
    public JavarestparameterModelManager createHypermedia(JavarestmethodModel oJavarestmethodModel){
        JavarestparameterModelManager oJavarestparameterModelManager = new JavarestparameterModelManager();

        /* Create hypermedia links towards this specific restparameter resource. These must be GET and POST as it is prescribed in the meta-models.*/
        oJavarestparameterModelManager.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Get all restparameters of this restmethod", "GET", "Sibling"));
        oJavarestparameterModelManager.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Create a new restparameter", "POST", "Sibling"));

        /* Then calculate the relative path to any related resource of this one and add for each one a hypermedia link to the Linklist.*/
        String oRelativePath;
        oRelativePath = oApplicationUri.getPath().replaceAll("multirestparameterManager/", "multirestparameter/");
        Iterator<JavarestparameterModel> setIterator = oJavarestmethodModel.getSetOfJavarestparameterModel().iterator();
        while(setIterator.hasNext()){
            JavarestparameterModel oNextJavarestparameterModel = new JavarestparameterModel();
            oNextJavarestparameterModel = setIterator.next();
            oJavarestparameterModelManager.getlinklist().add(new HypermediaLink(String.format("%s%s/%d", oApplicationUri.getBaseUri(), oRelativePath, oNextJavarestparameterModel.getrestparameterId()), String.format("%s", oNextJavarestparameterModel.getparametername()), "GET", "Child", oNextJavarestparameterModel.getrestparameterId()));
        }

        /* Finally calculate the relative path towards the resources of which this one is related and add one hypermedia link for each one of them in the Linklist.*/
        this.oJavarestmethodModel = HibernateController.getHibernateControllerHandle().getrestmethod(this.oJavarestmethodModel);
        oRelativePath = String.format("resource/%d/%s", this.oJavarestmethodModel.getresource().getresourceId(), oApplicationUri.getPath().replaceAll("multirestparameterManager/", ""));
        int iLastSlashIndex = String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).lastIndexOf("/");
        oJavarestparameterModelManager.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).substring(0, iLastSlashIndex), "Delete the parent restmethod", "DELETE", "Parent"));
        oJavarestparameterModelManager.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).substring(0, iLastSlashIndex), "Get the parent restmethod", "GET", "Parent"));
        oJavarestparameterModelManager.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).substring(0, iLastSlashIndex), "Update the restmethod", "PUT", "Parent"));
        return oJavarestparameterModelManager;
    }
}
