/*
 * ARISTOSTLE UNIVERSITY OF THESSALONIKI
 * Copyright (C) 2015
 * Aristotle University of Thessaloniki
 * Department of Electrical & Computer Engineering
 * Division of Electronics & Computer Engineering
 * Intelligent Systems & Software Engineering Lab
 *
 * Project             : WSAT
 * WorkFile            : 
 * Compiler            : 
 * File Description    : 
 * Document Description: 
* Related Documents	   : 
* Note				   : 
* Programmer		   : RESTful MDE Engine created by Christoforos Zolotas
* Contact			   : christopherzolotas@issel.ee.auth.gr
*/


package eu.fp7.scase.wsat.restservice;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.CollectionTable;
import javax.persistence.ElementCollection;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import javax.persistence.Table;
import javax.persistence.Transient;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.annotations.ForeignKey;

import eu.fp7.scase.wsat.utilities.HypermediaLink;
import eu.fp7.scase.wsat.resource.JavaresourceModel;
import eu.fp7.scase.wsat.account.JavaaccountModel;


/* This class models the data of a restservice resource. It is enhanced with JAXB annotations for automated representation
parsing/marshalling as well as with Hibernate annotations for ORM transformations.*/
@XmlRootElement
@Entity
@Table(name="restservice")
public class JavarestserviceModel{


    /* There follows a list with the properties that model the restservice resource, as prescribed in the service CIM*/
		@Column(name = "wsdescription")
		private String wsdescription;

		@ElementCollection(fetch = FetchType.EAGER)
		@CollectionTable(name="restservicewskeywords", joinColumns=@JoinColumn(name="restserviceId"))
		@Column(name = "wskeywords")
		@ForeignKey(name = "fk_restservice_wskeywords")
		private Set<String> wskeywords = new HashSet<String>();

		@Column(name = "wsprovider")
		private String wsprovider;

		@Column(name = "wsname")
		private String wsname;

		@Column(name = "baseuri")
		private String baseuri;

		@Id
		@GeneratedValue
		@Column(name = "restserviceId")
		private int restserviceId;

		// The Linklist property holds all the hypermedia links to be sent back to the client
		@Transient
		private List<HypermediaLink> linklist = new ArrayList<HypermediaLink>();

		// This property models the Many to One relationship between two resources as it is defined by the Hibernate syntax below.
		@ManyToOne(fetch = FetchType.EAGER)
		@JoinColumn(name="accountId")
		@ForeignKey(name = "fk_account_restservice")
		private JavaaccountModel account;

		// This property models the One to Many relationship between two resources as it is defined by the Hibernate syntax below.
		@OneToMany(fetch = FetchType.EAGER, mappedBy="restservice",orphanRemoval=true)
		@OnDelete(action=OnDeleteAction.CASCADE)
		private Set<JavaresourceModel> SetOfJavaresourceModel = new HashSet<JavaresourceModel>();

    /* There follows a list of setter and getter functions.*/
	    public void setwsdescription(String wsdescription){
        	this.wsdescription = wsdescription;
    	}

	    public void setwskeywords(Set<String> wskeywords){
        	this.wskeywords = wskeywords;
    	}

	    public void setwsprovider(String wsprovider){
        	this.wsprovider = wsprovider;
    	}

	    public void setwsname(String wsname){
        	this.wsname = wsname;
    	}

	    public void setbaseuri(String baseuri){
        	this.baseuri = baseuri;
    	}

	    public void setrestserviceId(int restserviceId){
        	this.restserviceId = restserviceId;
    	}

	    public void setlinklist(List<HypermediaLink> linklist){
        	this.linklist = linklist;
    	}

		@XmlTransient
	    public void setaccount(JavaaccountModel account){
        	this.account = account;
    	}

		@XmlTransient
	    public void setSetOfJavaresourceModel(Set<JavaresourceModel> SetOfJavaresourceModel){
        	this.SetOfJavaresourceModel = SetOfJavaresourceModel;
    	}

	    public String getwsdescription(){
        	return this.wsdescription;
    	}

	    public Set<String> getwskeywords(){
        	return this.wskeywords;
    	}

	    public String getwsprovider(){
        	return this.wsprovider;
    	}

	    public String getwsname(){
        	return this.wsname;
    	}

	    public String getbaseuri(){
        	return this.baseuri;
    	}

	    public int getrestserviceId(){
        	return this.restserviceId;
    	}

	    public List<HypermediaLink> getlinklist(){
        	return this.linklist;
    	}

	    public JavaaccountModel getaccount(){
        	return this.account;
    	}

	    public Set<JavaresourceModel> getSetOfJavaresourceModel(){
        	return this.SetOfJavaresourceModel;
    	}


    /* This function deletes explicitly any collections of this resource that are stored in the database 
    and iteratively does so for any subsequent related resources.
    NOTE: this function is needed to handle erroneous handling of cascade delete of some hibernate versions.*/
    public void deleteAllCollections(Session hibernateSession){

        Query query = hibernateSession.createSQLQuery(String.format("DELETE FROM %s where %sId = %d","restservicewskeywords".toLowerCase(),"restservice",this.getrestserviceId()));
        query.executeUpdate();

        Iterator<JavaresourceModel> JavaresourceModelIterator = SetOfJavaresourceModel.iterator();
        while(JavaresourceModelIterator.hasNext()){
            JavaresourceModelIterator.next().deleteAllCollections(hibernateSession);
        }
    }
}
